##########################
#
# analyse_glob_birds_data.r
#
# analysis for the collinearity paper, global birds alpha diversity case study
#
# script handles the analysis of the global birds alpha diversity and prepare figures
# preprocessing of the data is handled in data_preprocessing_complete.r
# content:
#		
#   1) load functions, data, creat birdsquad.15
#   2) diagonstics
#   3) analysis
#   	3.1 select07
# 		3.2 segreg
# 		3.3 PPLS
# 		3.4 LRR
# 		3.5 ivif.centred
# 		3.6 GLM
# 		3.7 GAM
# 		3.8 CWR
# 		3.9 varcluste.centred
# 		3.10 BRT
#   4) analysis across mehtods
#   5) forecast by changing area coverd by IUCN protected areas
#
# author: sven.lautenbach@ufz.de
# date: 05.08.09
# last modified: 01.09.2009
#
#
#########################

####
# data description
#
#
# Data from the CIA the world factbook
# (https://www.cia.gov/library/publications/the-world-factbook/)
# ---------------------------------------
# GDP-ag  GDP - composition by sector 		Part of GDP in agriculture	[%]
#	GDP-in	Part of GDP in industry	[%]
# GDP     purchasing power parity (ppp)	GDP-ppp		2007 US $
#
# Data from the Human Development Reports
# (http://hdr.undp.org/en/statistics/data/)
# -----------------------------------------
# Human development index 1975 - 2005	HDI_75_05	Rank of the country by the average HDI from 1975 till 2005	[-]
#
# data from World Resources Institute
# ( http://earthtrends.wri.org/searchable_db/index.php?action=select_countries&theme=7&variable_ID=139)
# additinonal data from
#  COLIN R.TRAINOR and THOMAS SOARES (2004), Birds of Atauro Island, Timor-Leste (East Timor) , Forktail 20
#  Checklist of the birds of Mayotte (2007), ABC & Bob Dowsett 2007
#  http://www.fatbirder.com
# -------------------------
# bird Birdspecies 	number of bird species	alpha diversity
#
# data from Worldwide Governance Indicators, 1996-2007
# (  http://info.worldbank.org/governance/wgi/index.asp)
# ----------------------------------------------------------
# con_cor Control of Corruption		extent to which public power is exercised for private gain, including petty and grand forms of corruption, as well as "capture" of the state by elites and private interests.
# gov_eff Government Effectiveness		quality of public services, quality of civil service and the degree of its independence from political pressures, quality of policy formulation and implementation, and the credibility of the government's commitment to such policies
# pol_sta Political Stability and Absence of Violence/Terrorism		perceptions of the likelihood that the government will be destabilized or overthrown by unconstitutional or violent means, including domestic violence and terrorism
# reg_qua Regulatory Quality		ability of the government to formulate and implement sound policies and regulations that permit and promote private sector development
# rul_law Rule of Law		measures the extent to which agents have confidence in and abide by the rules of society, in particular the quality of contract enforcement, the police, and the courts, as well as the likelihood of crime and violence
# voi_acc Voice & Accountability		measures the extent to which country's citizens are able to participate in selecting their government, as well as freedom of expression, freedom of association, and a free media
#
# data from Global Landcover 2000
# (  http://bioval.jrc.ec.europa.eu/products/glc2000/glc2000.php)
# -----------------------------------
# forest Forest		area covered by forest	[%]
# shrub Shrub mosaic		area covered by shrub or herbaceous areas	[%]
# arbl Cultivated and managed areas		area covered by  cultivated or managed areas	[%]
# urban Artificial areas		area covered by artificial areas	[%]
#
# World Database On Protected Areas (WDPA)
# ----------------------------------------------
# IU_cou National sites with IUCN Categories I-VI		Number of areas per country	[-]
#	IU_are	- Percentage of area protected 	[%]
#
# National Geophysixal Data Center (NGDC)
# (  http://www.wdpa.org/)
# -----------------------------------
# elev elevation		mean elevation per country 	[m a.m.s.l.]
# slop slope		mean slope	[%]
#
# Worldclim
# (http://www.wdpa.org/)
# -----------------
# ann_pre Annual precipitation			[mm/year]
# ann_tem Annual Mean temperature			C
# pre_sea Precipitation seasonality		coefficient of variation
# tem_sea	Temperature seasonality	standard deviation *100
#
#
########


require(mgcv)
require(car)
require(dr)
require(gbm)
require(Hmisc)
require(penalized)
require(perturb)
require(ppls)
require(randomForest)
require(RColorBrewer)


###
# 1) load functions and rename birdsquad.15 (should have been done in preprocessing...)
###
path <- "Y:/Gruppen/cle/Projekte/2007_DFG_collinearity/AA_paper&appendix/"
source(paste(sep="", path, "COLL_allfunctions.r"))
source(paste(sep="", path, "brtfunctions.r"))
#source(paste(sep="", path, "ivif.r"))
source(paste(sep="", path, "marsHelpers.r"))
source(paste(sep="", path, "diagnostics.r"))
#source(paste(sep="", path, "OSCAR_functions.r"))


dataFolder <- "Y:/Gruppen/cle/Projekte/2007_DFG_collinearity/AD_casestudies/global_species_distribution/data/"
load(file=paste(sep="", dataFolder, "birds_complete.Rdata"))
source(paste(sep="", dataFolder, "../analysis/helpers.r"))
set.seed(42)
###
# Make birdsquad names similar to names from formula to allow the use of importancePerVariable
###

f.rf <- randomForest(x=birds.df[,-1], y=birds.df[,1], ntree=1000, mtry=20, importance=TRUE)
varImpPlot(f.rf, type=2, main="Random Forest")	### 
#importance(f.rf, type=2)

########################### randomForest selection and order by importance
rf.idx<-order(f.rf$importance[,2], decreasing=TRUE)
cbind(importance(f.rf, type=2)[rf.idx], rownames(importance(f.rf))[rf.idx])
birds.df <- birds.df[,c(1,rf.idx+1)]
names(birds.df)[1] <- "bird"

###
#  create df with quadratic terms for the 15 most important variables
###
# use only the 15th most important variables for quadratic terms
birdsquad.15 <- cbind(birds.df[,1:16], birds.df[,2:16]^2)
#names(birdsquad.15) <- c(names(birds.df[,1:16]), paste(names(birds.df[,2:16]), ".2", sep=""))
names(birdsquad.15) <- c(names(birds.df[,1:16]), paste("I(", names(birds.df[,2:16]), "^2)", sep=""))


####
# 3) run diagnostics
###

colDiag <- colDiagnostics(birds.df[,-1], birds.df[,1], plot.hist = FALSE, silent=FALSE)
summary(colDiag)
colDiag$vif
colDiag$ivif.proxys
(pcaCluster <- PCAclusteringforward(birds.df[,-1]))
#cluster:hoeffding-ward:centred
varcluster(birds.df[,-1])
###
# 4) analysis
###
# 4.1 select07
# 4.2 segreg
# 4.3 PPLS
# 4.4 LRR
# 4.5 ivif.centred
# 4.6 GLM
# 4.7 GAM
# 4.8 CWR
# 4.9 varcluste.centred
# 4.10 BRT

#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

pdffile= paste(sep="", dataFolder, "../importance.pdf")
pdf(file=pdffile)

op<-par(mfrow=c(3,5), mar=c(3,3,1,1))
#for(aCol in (1:length(names(birds.df[,-1]))+1) )
for(aCol in (2:16) )
{
	plot(birds.df$bird ~ birds.df[,aCol], ylab="Alpha div birds", pch=16, col=rgb(.4,.5,.6,.75), log="y" )
	legend("topright", legend=names(birds.df)[aCol], bty="n", cex=.9)
	lines(lowess(birds.df$bird ~ birds.df[,aCol], f=1), col="grey80", lwd=2)
}
par(mfrow=c(1,1), mar=c(5.1,4.1, 4.1, 2.1))

biplot(prcomp(birds.df[,-1]), cex=c(.001, .85), col=c("black", "red"))#, xlim=c(-0.2,0.2), ylim=c(-0.15, 0.15))


########################################
# 1 select07

select07.vars <- select07(X=birds.df[,-1], y=birds.df[,1], family="gaussian", univar="gam", threshold=0.7, method="pearson")
select07.vars <- cbind.data.frame(y=birds.df[,1], select07.vars)
f <- formula.maker(select07.vars, quadratic=T, interactions=T)

###
# 1.1 select07.BIC
fm.select07.bic <- step(lm(f, data=select07.vars), k=log(nrow(select07.vars)), trace=FALSE)
summary(fm.select07.bic)
(anova.07.bic <- Anova  (fm.select07.bic, type="III")  )
# adj R^2: 0.63
idx.resid <- which(row.names(anova.07.bic) %in% c("(Intercept)","Residuals"))


theAnov <-anova.07.bic[-idx.resid,]
imp <- (theAnov )$"Sum Sq"
rel.imp <- abs(imp)/sum(abs(imp))
imp.Select07 <- importancePerVariable(theNames= row.names(theAnov) , theRelImp = rel.imp)
names(imp.Select07)[2] <-"rel.imp.select07"

idx <- order(theAnov$"Sum Sq", decreasing=FALSE)
dotchart(x=imp[idx[1:20]], labels=row.names(theAnov)[idx], xlab="Sum Sq", main="Select 07 BIC selection")
cor(predict(fm.select07.bic, newdata=birds.df[,-1]), birds.df[,1]) # .76
length(coefficients(fm.select07.bic)) - 1 # 28 but contains 3 non-significant terms


################################################################################

# 2 segreg
# 2.1 seqreg BIC
seqreg.obj.bic <- step.seqreg(data=birds.df[,1:16], y.col=1, k=log(nrow(birds.df)), quadratic=T, interactions=T, family="gaussian", univar="gam", sequence=NULL)
seqreg.obj.bic$model

summary.lm(seqreg.obj.bic$model)
plot.lm(seqreg.obj.bic$model) # -> rownames 36,91, 159, (200)
all.df[which(rownames(all.df) %in% c(36,91, 159, 200)),] # Bermuda, Gibraltar, Maledives, (Singapoure)

birds.seqreg.trans<- predict.seqreg(seqreg.obj.bic, newdata=birds.df)
cor(predict(seqreg.obj.bic$model, newdata=birds.seqreg.trans), birds.df[,1])

(anova.seqrec.bic <- Anova(seqreg.obj.bic$model, type=3))

theAnov <-anova.seqrec.bic
idx <- order(theAnov$"LR", decreasing=FALSE)

imp <- theAnov$"LR"
rel.imp <- abs(imp)/sum(abs(imp))
imp.Seqreg <- importancePerVariable(theNames= row.names(theAnov) , theRelImp = rel.imp)
names(imp.Seqreg)[2] <-"rel.imp.Seqreg"


dotchart(x=imp[idx], labels=row.names(anova.seqrec.bic)[idx], xlab="Deviance", main="Seqreg selection")


################################################################################

# 3 PPLS

#optimisation, grid search:
vals <- expand.grid(nk=1:20, lambda=seq(0, 100, by=5))
## functions_carsten --> pplsoptim
ores <- apply(vals, 1, pplsoptim, birdsquad.15)          ### runs really long
bestparms <- as.numeric(vals[which.max(ores),])

## functions_gudrun --> train.ppls
f.ppls <- train.ppls(birdsquad.15, nk=bestparms[1], lambda=bestparms[2])
# this step in the analysis reduces the number of components:
fit <- NS <- 1:ncol(f.ppls$ppls$coefficients)
for (i in NS){
      pred.birds.ppls <- f.ppls$transf.data %*% f.ppls$ppls$coefficients[,i]
      fit[i] <- cor(pred.birds.ppls, birdsquad.15[,1])^2
}

## functions_carsten --> AICc.r2
n.comp <- which.min(AICc.r2(fit, 100, NS))
f.ppls$comps <- n.comp # derived number of components is stored

n.comp <- which.min(AICc.r2(fit, 100, NS)) #calculates AIC for all fits and finds most parsimonious model
f.ppls$comps <- n.comp # derived number of components is stored


# 3.1 PPLS BIC
n.comp.bic <- which.min(BIC.r2(fit, 100, NS))
f.ppls.bic <- f.ppls
f.ppls.bic$comps <- n.comp.bic

# There is, to me, no obvious way to extract importances of the different variables from this model object.
# Maybe like this:
imps <- rowSums(abs(t(t(f.ppls$transf.data) %*% f.ppls$Xtrain) %*% (f.ppls$ppls$coefficients)))
rel.imp <- abs(imps)/sum(abs(imps))
imp.PPLS <- importancePerVariable(theNames= names(imps) , theRelImp = rel.imp)
names(imp.PPLS)[2] <-"rel.imp.PPLS"

idx <- order(imps, decreasing=FALSE)
dotchart(x=imps[idx] , labels=names(imps)[idx], xlab="importance ???", main="PPLS selection")

cor(f.ppls.bic$transf.data %*% f.ppls.bic$ppls$coefficients[,n.comp.bic], birdsquad.15[,1]) # .87

################################################################################

# 4 LRR
## functions_gudrun.r --> train.lrr
f.lrr <- train.lrr(birdsquad.15)
f.lrr$coef   # load on Latent Root variables
# To extract the importance, we have to multiply coefficients with loadings:
# To do so, first find which axes are still in the LRR:
used.axes <- as.numeric(substring(names(f.lrr$coef), first=6))
# Now, recognise that loadings and coefficients should all be positive (otherwise loadings may trade-off and therefore a variable important on 2 axes may actually show as irrelevant, if these two axes are pointing in different directions):
(imp <- t(abs(f.lrr$LOAD[, used.axes])  %*%  abs(f.lrr$coef)))
colnames(f.lrr$Xtrain)

rel.imp <- abs(imp)/sum(abs(imp))
imp.LRR <- importancePerVariable(theNames= colnames(f.lrr$Xtrain) , theRelImp = rel.imp)
names(imp.LRR)[2] <-"rel.imp.LRR"

idx <- order(imp, decreasing=FALSE)
dotchart(x=imp[idx] , labels=colnames(f.lrr$Xtrain)[idx], xlab="Loading * coeff", main="LRR selection")
cor(pred.lrr(f.lrr, newdata=birdsquad.15), birdsquad.15[,1]) #.53



################################################################################

# 5 ivif.centred

ivif.clust <- ivif(x=birds.df[,-1], y=birds.df[,1])$names.grouped

ivif.clust.vars <- CollClustReduction(X=birds.df[,-1], y=birds.df[,1], clusters=ivif.clust, varselect="centred") #'centred', 'explained' or 'summarised'

names(ivif.clust.vars)[1] <-  "bird"
f <- formula.maker(ivif.clust.vars, quadratic=T, interaction=T)
# 5.1 ivif.centred.BIC
fm.ivif.clust.bic <- step(lm(f, data=birds.df), k=log(nrow(birds.df)), trace=FALSE)

summary(fm.ivif.clust.bic) #
Anova (fm.ivif.clust.bic, type="III")
#plot (fm.ivif.clust.bic)

# add main effects to be in accordance with the principle of marginality
#...

( anova.ivif.cent.bic <- Anova (fm.ivif.clust.bic, type="III") )

idx.resid <- which(row.names(anova.ivif.cent.bic)%in% c("Residuals", "(Intercept)"))

theAnov <-anova.ivif.cent.bic[-idx.resid,]

imp <- (theAnov)$"Sum Sq"
rel.imp <- abs(imp)/sum(abs(imp))
imp.ivif.centr <- importancePerVariable(theNames= row.names(theAnov) , theRelImp = rel.imp)
names(imp.ivif.centr)[2] <-"rel.imp.ivif.centr"

idx <- order(theAnov$"Sum Sq", decreasing=FALSE)
dotchart(x=imp [idx], labels=row.names(theAnov)[idx], xlab="Sum Sq", main="iVIF.centered.bic selection")
cor(predict(fm.ivif.clust.bic, newdata=birds.df[,-1]), birds.df[,1]) # .4
length(coefficients(fm.ivif.clust.bic)) -1


################################################################################

# 6 GLM

f <- formula.maker(birds.df[,1:16], quadratic=TRUE, interactions=TRUE)
# 6.1 GLM BIC
f.glm.bic <- step(lm(f, data=birds.df), k=log(nrow(birds.df)), trace=FALSE)

summary(f.glm.bic)# adj R^2: .54
(anova.glm.bic <- Anova(f.glm.bic, type="III"))
#plot(f.glm.bic) # 228, 56, 108, 39

theAnov <- anova.glm.bic
idx.resid <- which(row.names(theAnov )%in% c("(Intercept)", "Residuals"))
theAnov <- theAnov[-idx.resid,]
idx <- order((theAnov)$"Sum Sq", decreasing=TRUE) # order the variables by importance
# get the standardized importance
imp <- (theAnov )$"Sum Sq"
rel.imp <- imp /sum(imp)
cbind(row.names(theAnov ), rel.imp)
dotchart(x=imp[idx[1:15]], labels=row.names(theAnov )[idx[1:15]], xlab="Rel. importance", main="GLM BIC selection")
# goodness of fit
cor(predict(f.glm.bic, newdata=birds.df[,-1]), birds.df[,1])#.78
# number of variables in full model
length(coefficients(f.glm.bic)) -1 # 35  -> Olalah! Step can't remove variables due to collinearity
# next step would be to delete variables manually

imp.GLM <- importancePerVariable(theNames= row.names(theAnov) , theRelImp = rel.imp)
names(imp.GLM)[2] <-"rel.imp.GLM"

# manual selection
#f.glm.bic.man <- update(f.glm.bic, "~. - IU_area:GDP_ppp - shrub:HDI - shrub:GDP_ag - urban:GDP_ag -urban:forest - I(HDI^2) - urban:HDI - GDP_ag ")
#summary(f.glm.bic.man)
#anova(f.glm.bic.man)

################################################################################

# 7 GAM

f <- as.formula(paste("birds.df[,1] ~", paste("s(", colnames(birds.df[,-1]), ", bs='cs', k=3)", collapse="+", sep="")))
fm.gam <- gam(f, data=birds.df, family=gaussian, gamma=1 )
summary(fm.gam) # R-sq.(adj) =  0.652
anova.gam <- anova(fm.gam)

theAnov <- anova.gam
idx <- order(theAnov$s.table[,3], decreasing=FALSE)
imp <- theAnov$s.table[,3]
rel.imp <- abs(imp)/sum(abs(imp))
cbind(row.names(theAnov ), rel.imp)
# get row.names without the s(..) parts
theNames <- character(length(imp))
i <- 0
for(varName in row.names(theAnov$s.table ))
{
	i <- i +1
	theNames[i] <- strsplit(strsplit( varName, "\\(")[[1]][2] , "\\)")[[1]][1]
}

imp.GAM <- data.frame(cbind(var.names=theNames[idx], data.frame(rel.imp.GAM=as.numeric(rel.imp[idx]))))

dotchart(x=rel.imp[idx] , labels=row.names(theAnov )[idx], xlab="F value", main="GAM selection")
cor(predict(fm.gam, newdata=birds.df[,-1]), birds.df[,1]) # .59
# number of vars after removing terms with coefficients shrinked to zero -> 11
# two insignificant terms
# -> 11 significant vars
par(mfrow=c(3,3))
plot(fm.gam)
par(mfrow=c(1,1))


################################################################################

# 8 CWR
W <- giveWeightsVif(X=as.matrix(birds.df[,2:10]), cut=0)#, expon=2) # takes quite a while!
f <- formula.maker(birds.df[,1:10], quadratic=T, interaction=T)
f.cwr <- step(lm(f, weights=W, data=cbind.data.frame(birds.df, W=W)), trace=F)

summary(f.cwr)
Anova(f.cwr, type="III")

# 8.1 CWR BIC

f.cwr.bic <- step(lm(f.cwr, data=cbind.data.frame(birds.df, W=W)), k=log(nrow(birds.df)), trace=FALSE)

summary(f.cwr.bic)

(anova.cwr.bic <- Anova(f.cwr.bic, type="III"))
idx.resid <- which(row.names(anova.cwr.bic) %in% c("(Intercept)", "Residuals"))

theAnov <-anova.cwr.bic[-idx.resid,]
idx.resid <- which(row.names(theAnov)=="Residuals")

imp <- (theAnov)$"Sum Sq"
rel.imp <- abs(imp)/sum(abs(imp))
imp.CWR <- importancePerVariable(theNames= row.names(theAnov), theRelImp = rel.imp)
names(imp.CWR)[2] <-"rel.imp.CWR"

idx <- order(theAnov$"Sum Sq", decreasing=FALSE)
dotchart(x=imp[idx], labels=row.names(theAnov)[idx], xlab="Sum Sq", main="CWR BIC selection")
cor(predict(f.cwr.bic, newdata=birds.df), birds.df[,1]) # .64
length(coefficients(f.cwr)) -1
################################################################################

# 9 varclust.centre

clusters <- varcluster(birds.df[,-1], threshold=.1)
cluster.vars <- CollClustReduction(X=birds.df[,-1], y=birds.df[,1], clusters=clusters, varselect="centred")
colnames(cluster.vars)[1] <- "bird"
f <- formula.maker(cluster.vars, quad=T, int=T)
fm.cluster <- step(lm(f, data=birds.df), k=log(nrow(birds.df)))
summary(fm.cluster)

#plot(fm.cluster)

(anova.cluster <- Anova(fm.cluster, type="III"))
idx.resid <- which(row.names(anova.cluster)%in% c("(Intercept)", "Residuals"))

theAnov <-anova.cluster[-idx.resid,]
imp <- (theAnov )$"Sum Sq"
rel.imp <- abs(imp)/sum(abs(imp))
imp.Varclust<- importancePerVariable(theNames= row.names(theAnov) , theRelImp = rel.imp)
names(imp.Varclust)[2] <-"rel.imp.Varclust"

idx <- order(theAnov$"Sum Sq", decreasing=FALSE)
dotchart(x=theAnov$"Sum Sq" [idx[1:15]], labels=row.names(theAnov)[idx], xlab="Sum Sq", main="varclust selection")
cor(predict(fm.cluster, newdata=birds.df), birds.df[,1]) #.81
length(coefficients(fm.cluster)) -1
################################################################################

# 10 BRT
# does not always run through, might be necessary to start BRT several times
# might be also worth comparing the results of the different runs

birds.df2 <- birds.df  # BRT deletes file somewhere!
f.brt <- try(gbm.step(data=birds.df2, gbm.x = 2:23, gbm.y = 1, family = "gaussian",  tree.complexity = 5, learning.rate = 0.01, bag.fraction = 0.8, n.trees=20, n.folds=5, verbose=TRUE, silent=FALSE, plot.main=TRUE, n.minobsinnode=5, tolerance.method = "fixed"))


gbm.plot(f.brt) # yields partial plot for each variable, ordered by importance; note: there are two (2) graphics windows open!!
summary(f.brt)

try(cor(gbm.predict.grids(f.brt, new.dat=birds.df2, eval.data, want.grids = FALSE, sp.name = "preds"), birds.df2[,1])) #.98

imp.BRT <- summary(f.brt)
imp.BRT$rel.inf <- imp.BRT$rel.inf/100
names(imp.BRT) <- c("var.names", "rel.imp.BRT")

#########################
# 11 lasso
f <- formula.maker(birds.df, quadratic=T, interaction=T)
lassosets <- optL1(response=birds.df[,1], penalized=f, minlambda1=0, maxlambda1=100, model="linear", fold=10, standardize=TRUE, trace=FALSE, data=birds.df) # optimise parameters
profile.lasso<- profL1(response=birds.df[,1], penalized=f, minlambda1=0, maxlambda1=100, model="linear", fold=10, standardize=TRUE, trace=FALSE, data=birds.df)
plot(profile.lasso$lambda, profile.lasso$cvl, type="l")
f.lasso <- penalized(response=birds.df[,1], penalized=f, lambda1=lassosets$lambda, model="linear", data=birds.df)
cor(predict.lasso(f.lasso, newdata=birds.df[,-1]), birds.df[,1]) # .79
plot(attr(f.lasso, "residuals") ~ attr(f.lasso, "fitted"))
plot(attr(f.lasso, "fitted") ~ birds.df$bird)
show(f.lasso)
coef(f.lasso, standardize=TRUE) 
linear.predictors(f.lasso)

################################
# 12 ridge
f <- formula.maker(birds.df, quadratic=T, interaction=T)
ridgesets <- optL2(response=birds.df[,1], penalized=f, minlambda2=1e-8, maxlambda2=100, model="linear", fold = 10, standardize=TRUE, trace=FALSE ,data=birds.df)
f.ridge <- penalized(response=birds.df[,1], penalized=f, lambda2=ridgesets$lambda, model="linear",data=birds.df)

cor(predict.lasso(f.ridge, newdata=birds.df[,-1]), birds.df[,1]) #.84
plot(attr(f.ridge, "residuals") ~ attr(f.lasso, "fitted"))
plot(attr(f.ridge, "fitted") ~ birds.df$bird)
plot(predict.lasso(f.ridge, newdata=birds.df[,-1]) ~ birds.df$bird)
show(f.ridge)
coef(f.ridge, standardize=TRUE)

dev.off()

#####################################
# 5) analysis across mehtods
####
# join importance data frames

imp.all <- merge(imp.GLM, imp.GAM, by="var.names", all=TRUE)
imp.all <- merge(imp.all, imp.Select07, by="var.names", all=TRUE)
imp.all <- merge(imp.all, imp.Seqreg, by="var.names", all=TRUE)
imp.all <- merge(imp.all, imp.PPLS, by="var.names", all=TRUE)
imp.all <- merge(imp.all, imp.LRR, by="var.names", all=TRUE)
imp.all <- merge(imp.all, imp.ivif.centr, by="var.names", all=TRUE)
imp.all <- merge(imp.all, imp.CWR, by="var.names", all=TRUE)
imp.all <- merge(imp.all, imp.Varclust, by="var.names", all=TRUE)
imp.all <- merge(imp.all, imp.BRT, by="var.names", all=TRUE)
#imp.all <- imp.all[,1:11]

# replace all NAs with zeros
(theNames <-names(imp.all)[-1])
for(aVar in theNames)
{
	print(aVar)
	idx <- which(is.na(imp.all[aVar]))
	if(length(idx)>0)
		imp.all[idx,][aVar] <- 0
}
imp.all

# sum the importance
imp.all$summed <- apply(imp.all[,-1], MARGIN=1, FUN=sum)
 apply(imp.all[,-1], MARGIN=2, FUN=sum)

save(imp.all, f.glm.bic, fm.gam, fm.select07.bic, seqreg.obj.bic, f.ppls, f.lrr, fm.ivif.clust.bic, f.cwr.bic, fm.cluster, f.brt, ores, file=paste(sep="",dataFolder, "importanceMethods.Rdata" ))
#load(file=paste(sep="",dataFolder, "importanceMethods.Rdata" ) )

###
# prepare some more or less nice figures
###


imp.cols <- as.data.frame(imp.all[,-c(1,ncol(imp.all))])
method.names <- unlist(lapply(names(imp.cols),  FUN= function(x) strsplit(x,"rel.imp.")[[1]][2]))
method.names[which(method.names=="ivif.centr")] <- "iVIF.centr"
method.names[which(method.names=="select07")] <- "Select07"
method.names[which(method.names=="Varclust")] <- "Hoeffding-ward.centr"


#barplot(c(as.matrix(imp.cols), rep(0,nrow(imp.all)) ) , col=c(cm.colors(6), rainbow(6), topo.colors(6), terrain.colors(5)), legend.text=imp.all$var.names, beside=FALSE, horiz=FALSE)#, names.arg=names(imp.cols), ""))


method.idx <- rev(c(1:3,7,9,4:6, 8,10))
#vars.idx <- 1:24
#vars.idx <- order(imp.all$rel.imp.BRT, decreasing=TRUE)
#vars.idx <- c(12, 10, 13, 17, 16, 4,  23, 11, 7, 8, 9, 2, 3, 20, 21,  14, 15, 5, 19, 1, 6, 18, 22 )
# col means
varMeans <- apply(imp.cols, MAR=1, FUN=mean)
cbind(levels(imp.all$var.names), round(varMeans,3))
vars.idx <- order(varMeans, decreasing=TRUE) # sort variables by mean importance

vIdxs <- function(varNames)
{
	n <- length(varNames)
	res <- rep(NA, n)
	for(i in 1:n)
	  res[i] <- which(imp.all$var.names[vars.idx] == varNames[i])
	return(res)
}
mIdx <- function(metName)
{ return(which(method.names[method.idx] == metName)) }


color <-  brewer.pal(9, "Blues")#"Spectral" )#
breaks <- c(seq(0,.4,.05), .5)
blueColors <- colorRampPalette(colors=color)

par(mar=c(6.1,8.1,4.1,2.1))
image(as.matrix(imp.cols), axes=FALSE,col=color, breaks = breaks )
axis(1, at = seq(0, 1, by = 1/(length(imp.all$var.names)-1)), labels=imp.all$var.names, las=2)
axis(2, at = seq(0, 1, by = 1/(length(names(imp.cols))-1)), labels=names(imp.cols), las=1)


require(lattice)
windows(width=12, height=6)

myPanel <- function(x,y,...){
	panel.levelplot(x,y,...)
	# ivif cluster
	ltext(x=vIdxs(c("tem_sea", "tem_y",   "forest",  "GDP_in",  "prc_y",   "IU_cou" )), y=mIdx("iVIF.centr"), labels="A", cex=1, col=c("grey80", "grey80"))
	ltext(x=vIdxs(c("urban",   "IU_area", "GDP_ppp", "HDI", "GDP_ag",  "reg_qua", "con_cor", "rul_law")), y=mIdx("iVIF.centr"), labels="B", cex=1, col=c("grey60"))
	ltext(x=vIdxs(c( "voi_acc", "pol_sta", "gov_eff" )), y=mIdx("iVIF.centr"), labels="C", cex=1, col=c("grey40"))
	

	# varclust
 	ltext(x=vIdxs(c("gov_eff", "pol_sta")), y=mIdx("Hoeffding-ward.centr"), labels="A", cex=1, col=c("grey80", "grey80"))
	ltext(x=vIdxs(c("rul_law", "con_cor", "reg_qua", "voi_acc")), y=mIdx("Hoeffding-ward.centr"), labels="B", cex=1, col=c("grey60"))
	ltext(x=vIdxs(c("tem_sea", "tem_y", "prc_y")), y=mIdx("Hoeffding-ward.centr"), labels="C", cex=1, col=c("grey40"))
	ltext(x=vIdxs(c("elev", "slope")), y=mIdx("Hoeffding-ward.centr"), labels="D", cex=1, col=c("grey20"))
  ltext(x=vIdxs(c("IU_cou", "IU_area")), y=mIdx("Hoeffding-ward.centr"), labels="E", cex=1, col=c("black"))
  ltext(x=vIdxs(c("GDP_ppp", "GDP_ag", "HDI")), y=mIdx("Hoeffding-ward.centr"), labels="F", cex=1, col=c("brown"))
}

levelplot(as.matrix(imp.cols[vars.idx ,method.idx]),  xlab="Variables", ylab="Methods", scales=list(x=list(at=1:22, labels=imp.all$var.names[vars.idx], rot=90), y=list(labels=method.names[method.idx] )), col.regions=c(blueColors(100)), panel=myPanel  )

apply(imp.cols, MAR=2, FUN=max)
apply(imp.cols, MAR=2, FUN=sd)
apply(imp.cols, MAR=2, FUN=mean) # -> strange a lot of the means are exactly the same
apply(imp.cols, MAR=2, FUN=median)
apply(imp.cols, MAR=2, FUN = function(x) sum(x==0))


barplot( varMeans[vars.idx], beside=TRUE, names.arg = imp.all$var.names[vars.idx], las=2, cex.names=.8)
boxplot((imp.cols), names=method.names, ylab="Relative importance")
boxplot(as.data.frame(t(imp.cols[vars.idx,])), names=imp.all$var.names[vars.idx], ylab="Relative importance", las=2, cex.axis=.8)


par(mfrow=c(2,5), mar=c(3,3,1,1))#, mar=c(3.1,3.1,3.1,2.1))
for(i in 1:ncol(imp.cols))
	hist(imp.cols[,i], main=method.names[i], breaks=seq(0,0.4,0.0125), col="grey", ylim=c(0,20), xlab="Rel. importance", ylab="")
par(mfrow=c(1,1), mar=c(5.1, 4.1, 4.1, 2.1))

op<-par(mfrow=c(3,5), mar=c(3,3,1,1))
for(aVar in as.character(imp.all$var.names[vars.idx][1:15]) )
{
	plot(birds.df$bird ~ unlist(birds.df[aVar]), ylab="Alpha div birds", pch=16, col=rgb(.4,.5,.6,.75) )
	legend("topright", legend=aVar, bty="n", cex=.9)
	lines(lowess(birds.df$bird ~ unlist(birds.df[aVar]), f=1), col="grey80", lwd=2)
}
par(mfrow=c(1,1), mar=c(5.1,4.1, 4.1, 2.1))

biplot(prcomp(birds.df[,-1]), cex=c(.001, .85), col=c("black", "red"))#, xlim=c(-0.2,0.2), ylim=c(-0.15, 0.15))
biplot(prcomp(birds.df[as.character(imp.all$var.names[vars.idx][1:15])]), cex=c(.001, .85), col=c("black", "red"))


###
# aggregate the rel importance for thematic groups of variables
###
(lu.vars <- which(imp.all$var.names %in% c("arbl", "forest", "shrub", "urban")))
(clim.vars <- which(imp.all$var.names %in% c("prc_sea", "prc_y", "tem_sea", "tem_y")))
(pol.vars <- which(imp.all$var.names %in% c("con_cor", "gov_eff", "pol_sta", "reg_qua", "rul_law", "voi_acc")))
(ter.vars <- which(imp.all$var.names %in% c("slope", "elev")))
(econ.vars <- which(imp.all$var.names %in% c("GDP_ag", "GDP_in", "GDP_ppp", "HDI")))
(prot.vars <- which(imp.all$var.names %in% c("IU_area", "IU_cou")))
# security check
sort(c(lu.vars, clim.vars, ter.vars, econ.vars, prot.vars, pol.vars))
agg.lu  <- apply(imp.all[lu.vars,-1],MARGIN=2,  sum )
agg.clim  <- apply(imp.all[clim.vars,-1],MARGIN=2,  sum )
agg.pol  <- apply(imp.all[pol.vars,-1],MARGIN=2,  sum )
agg.ter  <- apply(imp.all[ter.vars,-1],MARGIN=2,  sum )
agg.econ  <- apply(imp.all[econ.vars,-1],MARGIN=2,  sum )
agg.prot  <- apply(imp.all[prot.vars,-1],MARGIN=2,  sum )

imp.agg <- rbind(agg.lu, agg.clim, agg.pol , agg.ter, agg.econ, agg.prot)[,-11]
agg.lab <- c("lu", "clim", "policy", "terrain", "econ", "prot")

levelplot(as.matrix(imp.agg[ ,method.idx]),  xlab="Variables", ylab="Methods", scales=list(x=list(at=1:6, labels=agg.lab, rot=90), y=list(labels=method.names[method.idx] )), col.regions=c(blueColors(100))  )
